@@ -56,16 +56,12 @@ tag_cmp_by_line (gconstpointer a,
   gint          direction = GPOINTER_TO_INT (data);
   gint          rv;
   
-  if (t1->type & tm_tag_file_t || t2->type & tm_tag_file_t) {
-    rv = 0;
+  if (t1->line > t2->line) {
+    rv = +direction;
+  } else if (t1->line < t2->line) {
+    rv = -direction;
   } else {
-    if (t1->atts.entry.line > t2->atts.entry.line) {
-      rv = +direction;
-    } else if (t1->atts.entry.line < t2->atts.entry.line) {
-      rv = -direction;
-    } else {
-      rv = 0;
-    }
+    rv = 0;
   }
   
   return rv;
@@ -121,8 +117,8 @@ ggd_tag_sort_by_line_to_list (const GPtrArray  *tags,
                               gint              direction)
 {
   GList  *children = NULL;
-  guint   i;
-  TMTag  *el;
+  guint   i = 0;
+  TMTag  *el = NULL;
   
   g_return_val_if_fail (tags != NULL, NULL);
   g_return_val_if_fail (direction != 0, NULL);
@@ -152,17 +148,15 @@ ggd_tag_find_from_line (const GPtrArray  *tags,
                         gulong            line)
 {
   TMTag    *tag = NULL;
-  TMTag    *el;
-  guint     i;
+  TMTag    *el = NULL;
+  guint     i = 0;
   
   g_return_val_if_fail (tags != NULL, NULL);
   
   GGD_PTR_ARRAY_FOR (tags, i, el) {
-    if (! (el->type & tm_tag_file_t)) {
-      if (el->atts.entry.line <= line &&
-          (! tag || el->atts.entry.line > tag->atts.entry.line)) {
-        tag = el;
-      }
+    if (el->line <= line &&
+        (! tag || el->line > tag->line)) {
+      tag = el;
     }
   }
   
@@ -203,7 +197,7 @@ ggd_tag_find_at_current_pos (GeanyDocument *doc)
  */
 TMTag *
 ggd_tag_find_parent (const GPtrArray *tags,
-                     filetype_id      geany_ft,
+                     GeanyFiletypeID  geany_ft,
                      const TMTag     *child)
 {
   TMTag *tag = NULL;
@@ -211,38 +205,37 @@ ggd_tag_find_parent (const GPtrArray *tags,
   g_return_val_if_fail (tags != NULL, NULL);
   g_return_val_if_fail (child != NULL, NULL);
   
-  if (! child->atts.entry.scope) {
+  if (! child->scope) {
     /* tag has no parent, we're done */
   } else {
     gchar        *parent_scope = NULL;
     const gchar  *parent_name;
     const gchar  *tmp;
-    guint         i;
-    TMTag        *el;
+    guint         i = 0;
+    TMTag        *el = NULL;
     const gchar  *separator;
     gsize         separator_len;
     
     /* scope is of the form a<sep>b<sep>c */
-    parent_name = child->atts.entry.scope;
+    parent_name = child->scope;
     separator = symbols_get_context_separator (geany_ft);
     separator_len = strlen (separator);
     while ((tmp = strstr (parent_name, separator)) != NULL) {
       parent_name = &tmp[separator_len];
     }
     /* if parent have scope */
-    if (parent_name != child->atts.entry.scope) {
+    if (parent_name != child->scope) {
       /* the parent scope is the "dirname" of the child's scope */
-      parent_scope = g_strndup (child->atts.entry.scope,
-                                parent_name - child->atts.entry.scope -
+      parent_scope = g_strndup (child->scope,
+                                parent_name - child->scope -
                                   separator_len);
     }
     /*g_debug ("%s: parent_name = %s", G_STRFUNC, parent_name);
     g_debug ("%s: parent_scope = %s", G_STRFUNC, parent_scope);*/
     GGD_PTR_ARRAY_FOR (tags, i, el) {
-      if (! (el->type & tm_tag_file_t) &&
-          (utils_str_equal (el->name, parent_name) &&
-           utils_str_equal (el->atts.entry.scope, parent_scope) &&
-           el->atts.entry.line <= child->atts.entry.line)) {
+      if ((utils_str_equal (el->name, parent_name) &&
+           utils_str_equal (el->scope, parent_scope) &&
+           el->line <= child->line)) {
         tag = el;
       }
     }
@@ -273,8 +266,7 @@ static const struct {
   { tm_tag_variable_t,        "variable"  },
   { tm_tag_externvar_t,       "extern"    },
   { tm_tag_macro_t,           "define"    },
-  { tm_tag_macro_with_arg_t,  "macro"     },
-  { tm_tag_file_t,            "file"      }
+  { tm_tag_macro_with_arg_t,  "macro"     }
 };
 
 /**
@@ -358,35 +350,30 @@ ggd_tag_get_type_name (const TMTag *tag)
  */
 gchar *
 ggd_tag_resolve_type_hierarchy (const GPtrArray *tags,
-                                filetype_id      geany_ft,
+                                GeanyFiletypeID  geany_ft,
                                 const TMTag     *tag)
 {
+  TMTag *parent_tag;
   gchar *scope = NULL;
   
   g_return_val_if_fail (tags != NULL, NULL);
   g_return_val_if_fail (tag != NULL, NULL);
-  
-  if (tag->type & tm_tag_file_t) {
-    g_critical (_("Invalid tag"));
-  } else {
-    TMTag *parent_tag;
     
-    parent_tag = ggd_tag_find_parent (tags, geany_ft, tag);
-    scope = g_strdup (ggd_tag_get_type_name (tag));
-    if (parent_tag) {
-      gchar *parent_scope;
+  parent_tag = ggd_tag_find_parent (tags, geany_ft, tag);
+  scope = g_strdup (ggd_tag_get_type_name (tag));
+  if (parent_tag) {
+    gchar *parent_scope;
+    
+    parent_scope = ggd_tag_resolve_type_hierarchy (tags, geany_ft, parent_tag);
+    if (! parent_scope) {
+      /*g_debug ("no parent scope");*/
+    } else {
+      gchar *tmp;
       
-      parent_scope = ggd_tag_resolve_type_hierarchy (tags, geany_ft, parent_tag);
-      if (! parent_scope) {
-        /*g_debug ("no parent scope");*/
-      } else {
-        gchar *tmp;
-        
-        tmp = g_strconcat (parent_scope, ".", scope, NULL);
-        g_free (scope);
-        scope = tmp;
-        g_free (parent_scope);
-      }
+      tmp = g_strconcat (parent_scope, ".", scope, NULL);
+      g_free (scope);
+      scope = tmp;
+      g_free (parent_scope);
     }
   }
   
@@ -407,15 +394,14 @@ ggd_tag_find_from_name (const GPtrArray *tags,
                         const gchar     *name)
 {
   TMTag  *tag = NULL;
-  guint   i;
-  TMTag  *el;
+  guint   i = 0;
+  TMTag  *el = NULL;
   
   g_return_val_if_fail (tags != NULL, NULL);
   g_return_val_if_fail (name != NULL, NULL);
   
   GGD_PTR_ARRAY_FOR (tags, i, el) {
-    if (! (el->type & tm_tag_file_t) &&
-        utils_str_equal (el->name, name)) {
+    if (utils_str_equal (el->name, name)) {
       tag = el;
       break;
     }
@@ -440,12 +426,12 @@ ggd_tag_find_from_name (const GPtrArray *tags,
 GList *
 ggd_tag_find_children_filtered (const GPtrArray *tags,
                                 const TMTag     *parent,
-                                filetype_id      geany_ft,
+                                GeanyFiletypeID  geany_ft,
                                 TMTagType        filter)
 {
   GList  *children = NULL;
-  guint   i;
-  TMTag  *el;
+  guint   i = 0;
+  TMTag  *el = NULL;
   
   g_return_val_if_fail (tags != NULL, NULL);
   g_return_val_if_fail (parent != NULL, NULL);
@@ -477,7 +463,7 @@ ggd_tag_find_children_filtered (const GPtrArray *tags,
 GList *
 ggd_tag_find_children (const GPtrArray *tags,
                        const TMTag     *parent,
-                       filetype_id      geany_ft)
+                       GeanyFiletypeID  geany_ft)
 {
   return ggd_tag_find_children_filtered (tags, parent, geany_ft, tm_tag_max_t);
 }
