@@ -387,7 +387,7 @@ get_setting_from_tag (GgdDocType     *doctype,
   gchar          *hierarchy;
   gint            nth_child;
   GPtrArray      *tag_array = doc->tm_file->tags_array;
-  filetype_id     geany_ft = FILETYPE_ID (doc->file_type);
+  GeanyFiletypeID geany_ft = FILETYPE_ID (doc->file_type);
   
   hierarchy = ggd_tag_resolve_type_hierarchy (tag_array, geany_ft, tag);
   /*g_debug ("type hierarchy for tag %s is: %s", tag->name, hierarchy);*/
@@ -529,7 +529,7 @@ ggd_insert_comment (GeanyDocument  *doc,
     tag_array = doc->tm_file->tags_array;
     tag = ggd_tag_find_from_line (tag_array, line + 1 /* it is a SCI line */);
   }
-  if (! tag || (tag->type & tm_tag_file_t)) {
+  if (! tag) {
     msgwin_status_add (_("No valid tag at line %d."), line);
   } else {
     if (get_config (doc, doc_type, &filetype, &doctype)) {
