@@ -22,7 +22,11 @@
 #include "config.h"
 #include <string.h>
 #include <gtk/gtk.h>
-#include <webkit/webkitwebview.h>
+#ifdef MARKDOWN_WEBKIT2
+# include <webkit2/webkit2.h>
+#else
+# include <webkit/webkitwebview.h>
+#endif
 #include <geanyplugin.h>
 #ifndef FULL_PRICE
 # include <mkdio.h>
@@ -296,6 +300,18 @@ pop_scroll_pos(MarkdownViewer *self)
   return popped;
 }
 
+#ifdef MARKDOWN_WEBKIT2
+static void
+on_webview_load_changed(MarkdownViewer  *self,
+                        WebKitLoadEvent  load_event,
+                        WebKitWebView   *web_view)
+{
+  /* When the webkit is done loading, reset the scroll position. */
+  if (load_event == WEBKIT_LOAD_FINISHED) {
+    pop_scroll_pos(self);
+  }
+}
+#else
 static void
 on_webview_load_status_notify(WebKitWebView *view, GParamSpec *pspec,
   MarkdownViewer *self)
@@ -309,6 +325,7 @@ on_webview_load_status_notify(WebKitWebView *view, GParamSpec *pspec,
     pop_scroll_pos(self);
   }
 }
+#endif
 
 gchar *
 markdown_viewer_get_html(MarkdownViewer *self)
@@ -388,13 +405,23 @@ markdown_viewer_update_view(MarkdownViewer *self)
     /* Connect a signal handler (only needed once) to restore the scroll
      * position once the webview is reloaded. */
     if (self->priv->load_handle == 0) {
+#ifdef MARKDOWN_WEBKIT2
+      self->priv->load_handle =
+        g_signal_connect_swapped(WEBKIT_WEB_VIEW(self), "load-changed",
+          G_CALLBACK(on_webview_load_changed), self);
+#else
       self->priv->load_handle =
         g_signal_connect_swapped(WEBKIT_WEB_VIEW(self), "notify::load-status",
           G_CALLBACK(on_webview_load_status_notify), self);
+#endif
     }
 
+#ifdef MARKDOWN_WEBKIT2
+    webkit_web_view_load_html(WEBKIT_WEB_VIEW(self), html, base_uri);
+#else
     webkit_web_view_load_string(WEBKIT_WEB_VIEW(self), html, "text/html",
       self->priv->enc, base_uri);
+#endif
 
     g_free(base_uri);
     g_free(html);
