@@ -17,11 +17,10 @@
  *  along with this program. If not, see <http://www.gnu.org/licenses/>.
  */
 
-#include "geanyplugin.h"
+#include <geanyplugin.h>
 
 extern GeanyPlugin *geany_plugin;
 extern GeanyData *geany_data;
-extern GeanyFunctions *geany_functions;
 
 typedef struct _MenuItem MenuItem;
 #define FRAME_ARGS '0' + (int) strlen(thread_id) - 1, thread_id, frame_id
@@ -29,7 +28,6 @@ typedef struct _MenuItem MenuItem;
 #include "break.h"
 #include "conterm.h"
 #include "debug.h"
-#include "gtk216.h"
 #include "inspect.h"
 #include "local.h"
 #include "memory.h"
