@@ -470,7 +470,12 @@ void sc_speller_add_word(const gchar *word)
 	g_return_if_fail(sc_speller_dict != NULL);
 	g_return_if_fail(word != NULL);
 
+#ifdef HAVE_ENCHANT_1_5
+	/* enchant_dict_add() is available since Enchant 1.4 */
+	enchant_dict_add(sc_speller_dict, word, -1);
+#else
 	enchant_dict_add_to_pwl(sc_speller_dict, word, -1);
+#endif
 }
 
 gboolean sc_speller_dict_check(const gchar *word)
@@ -518,7 +523,18 @@ void sc_speller_reinit_enchant_dict(void)
 	if (sc_speller_dict != NULL)
 		enchant_broker_free_dict(sc_speller_broker, sc_speller_dict);
 
-#if HAVE_ENCHANT_1_5
+#ifdef HAVE_ENCHANT_2_0
+	#define ENCHANT_CONFIG_ENV_NAME "ENCHANT_CONFIG_DIR"
+	/* set custom configuration path for enchant (Enchant will look for dictionaries there) */
+	if (! EMPTY(sc_info->dictionary_dir))
+	{
+		g_setenv(ENCHANT_CONFIG_ENV_NAME, sc_info->dictionary_dir, TRUE);
+	}
+	else
+	{
+		g_unsetenv(ENCHANT_CONFIG_ENV_NAME);
+	}
+#elif HAVE_ENCHANT_1_5
 	{
 		const gchar *old_path;
 		gchar *new_path;
@@ -569,8 +585,21 @@ void sc_speller_reinit_enchant_dict(void)
 }
 
 
+static void log_enchant_version(void)
+{
+#ifdef HAVE_ENCHANT_2_0
+	const gchar *enchant_version = enchant_get_version();
+#else
+	const gchar *enchant_version = "1.6 or older";
+#endif
+
+	g_debug("Initializing Enchant library version %s", enchant_version);
+}
+
+
 void sc_speller_init(void)
 {
+	log_enchant_version();
 	sc_speller_broker = enchant_broker_init();
 
 	sc_speller_reinit_enchant_dict();
@@ -631,6 +660,37 @@ gboolean sc_speller_is_text(GeanyDocument *doc, gint pos)
 			}
 			break;
 		}
+		case SCLEX_ASCIIDOC:
+		{
+			switch (style)
+			{
+				case SCE_ASCIIDOC_DEFAULT:
+				case SCE_ASCIIDOC_STRONG1:
+				case SCE_ASCIIDOC_STRONG2:
+				case SCE_ASCIIDOC_EM1:
+				case SCE_ASCIIDOC_EM2:
+				case SCE_ASCIIDOC_HEADER1:
+				case SCE_ASCIIDOC_HEADER2:
+				case SCE_ASCIIDOC_HEADER3:
+				case SCE_ASCIIDOC_HEADER4:
+				case SCE_ASCIIDOC_HEADER5:
+				case SCE_ASCIIDOC_HEADER6:
+				case SCE_ASCIIDOC_ULIST_ITEM:
+				case SCE_ASCIIDOC_OLIST_ITEM:
+				case SCE_ASCIIDOC_BLOCKQUOTE:
+				case SCE_ASCIIDOC_PASSBK:
+				case SCE_ASCIIDOC_COMMENT:
+				case SCE_ASCIIDOC_COMMENTBK:
+				case SCE_ASCIIDOC_LITERAL:
+				case SCE_ASCIIDOC_LITERALBK:
+				case SCE_ASCIIDOC_ATTRIB:
+				case SCE_ASCIIDOC_ATTRIBVAL:
+					return TRUE;
+				default:
+					return FALSE;
+			}
+			break;
+		}
 		case SCLEX_ASM:
 		{
 			switch (style)
@@ -647,6 +707,20 @@ gboolean sc_speller_is_text(GeanyDocument *doc, gint pos)
 			}
 			break;
 		}
+		case SCLEX_AU3:
+		{
+			switch (style)
+			{
+				case SCE_AU3_DEFAULT:
+				case SCE_AU3_COMMENT:
+				case SCE_AU3_COMMENTBLOCK:
+				case SCE_AU3_STRING:
+					return TRUE;
+				default:
+					return FALSE;
+			}
+			break;
+		}
 		case SCLEX_BASH:
 		{
 			switch (style)
@@ -715,6 +789,7 @@ gboolean sc_speller_is_text(GeanyDocument *doc, gint pos)
 				case SCE_PAS_COMMENT2:
 				case SCE_PAS_COMMENTLINE:
 				case SCE_PAS_STRING:
+				case SCE_PAS_STRINGEOL:
 				case SCE_PAS_CHARACTER:
 					return TRUE;
 				default:
@@ -738,6 +813,11 @@ gboolean sc_speller_is_text(GeanyDocument *doc, gint pos)
 				case SCE_C_CHARACTER:
 				case SCE_C_STRINGEOL:
 				case SCE_C_COMMENTLINEDOC:
+				case SCE_C_COMMENTDOCKEYWORDERROR:
+				case SCE_C_STRINGRAW:
+				case SCE_C_TRIPLEVERBATIM:
+				case SCE_C_PREPROCESSORCOMMENT:
+				case SCE_C_PREPROCESSORCOMMENTDOC:
 					return TRUE;
 				default:
 					return FALSE;
@@ -750,8 +830,10 @@ gboolean sc_speller_is_text(GeanyDocument *doc, gint pos)
 			{
 				case SCE_COFFEESCRIPT_CHARACTER:
 				case SCE_COFFEESCRIPT_COMMENTBLOCK:
+				case SCE_COFFEESCRIPT_COMMENTDOC:
 				case SCE_COFFEESCRIPT_COMMENTDOCKEYWORD:
 				case SCE_COFFEESCRIPT_COMMENTDOCKEYWORDERROR:
+				case SCE_COFFEESCRIPT_COMMENTLINE:
 				case SCE_COFFEESCRIPT_COMMENTLINEDOC:
 				case SCE_COFFEESCRIPT_STRING:
 				case SCE_COFFEESCRIPT_STRINGEOL:
@@ -787,6 +869,10 @@ gboolean sc_speller_is_text(GeanyDocument *doc, gint pos)
 				case SCE_D_STRINGEOL:
 				case SCE_D_CHARACTER:
 				case SCE_D_COMMENTLINEDOC:
+				case SCE_D_COMMENTDOCKEYWORD:
+				case SCE_D_COMMENTDOCKEYWORDERROR:
+				case SCE_D_STRINGB:
+				case SCE_D_STRINGR:
 					return TRUE;
 				default:
 					return FALSE;
@@ -864,6 +950,25 @@ gboolean sc_speller_is_text(GeanyDocument *doc, gint pos)
 				case SCE_B_STRING:
 				case SCE_B_STRINGEOL:
 				case SCE_B_CONSTANT:
+				case SCE_B_COMMENTBLOCK:
+				case SCE_B_DOCLINE:
+				case SCE_B_DOCBLOCK:
+				case SCE_B_DOCKEYWORD:
+					return TRUE;
+				default:
+					return FALSE;
+			}
+			break;
+		}
+		case SCLEX_GDSCRIPT:
+		{
+			switch (style)
+			{
+				case SCE_GD_DEFAULT:
+				case SCE_GD_COMMENTLINE:
+				case SCE_GD_STRING:
+				case SCE_GD_STRINGEOL:
+				case SCE_GD_COMMENTBLOCK:
 					return TRUE;
 				default:
 					return FALSE;
@@ -880,8 +985,10 @@ gboolean sc_speller_is_text(GeanyDocument *doc, gint pos)
 				case SCE_HA_COMMENTBLOCK2:
 				case SCE_HA_COMMENTBLOCK3:
 				case SCE_HA_STRING:
+				case SCE_HA_STRINGEOL:
 				case SCE_HA_CHARACTER:
 				case SCE_HA_DATA:
+				case SCE_HA_LITERATE_COMMENT:
 					return TRUE;
 				default:
 					return FALSE;
@@ -945,12 +1052,29 @@ gboolean sc_speller_is_text(GeanyDocument *doc, gint pos)
 			}
 			break;
 		}
+		case SCLEX_JULIA:
+		{
+			switch (style)
+			{
+				case SCE_JULIA_DEFAULT:
+				case SCE_JULIA_COMMENT:
+				case SCE_JULIA_STRING:
+				case SCE_JULIA_STRINGINTERP:
+				case SCE_JULIA_DOCSTRING:
+				case SCE_JULIA_STRINGLITERAL:
+					return TRUE;
+				default:
+					return FALSE;
+			}
+			break;
+		}
 		case SCLEX_LATEX:
 		{
 			switch (style)
 			{
 				case SCE_L_DEFAULT:
 				case SCE_L_COMMENT:
+				case SCE_L_COMMENT2:
 					return TRUE;
 				default:
 					return FALSE;
@@ -1066,6 +1190,7 @@ gboolean sc_speller_is_text(GeanyDocument *doc, gint pos)
 				case SCE_PO_MSGID_TEXT:
 				case SCE_PO_MSGSTR_TEXT:
 				case SCE_PO_MSGCTXT_TEXT:
+				case SCE_PO_PROGRAMMER_COMMENT:
 					return TRUE;
 				default:
 					return FALSE;
@@ -1111,6 +1236,9 @@ gboolean sc_speller_is_text(GeanyDocument *doc, gint pos)
 				case SCE_P_TRIPLEDOUBLE:
 				case SCE_P_COMMENTBLOCK:
 				case SCE_P_STRINGEOL:
+				case SCE_P_FSTRING:
+				case SCE_P_FTRIPLE:
+				case SCE_P_FTRIPLEDOUBLE:
 					return TRUE;
 				default:
 					return FALSE;
@@ -1166,6 +1294,18 @@ gboolean sc_speller_is_text(GeanyDocument *doc, gint pos)
 			}
 			break;
 		}
+		case SCLEX_SMALLTALK:
+		{
+			switch (style)
+			{
+				case SCE_ST_DEFAULT:
+				case SCE_ST_STRING:
+				case SCE_ST_COMMENT:
+				default:
+					return FALSE;
+			}
+			break;
+		}
 		case SCLEX_SQL:
 		{
 			switch (style)
@@ -1177,6 +1317,9 @@ gboolean sc_speller_is_text(GeanyDocument *doc, gint pos)
 				case SCE_SQL_STRING:
 				case SCE_SQL_CHARACTER:
 				case SCE_SQL_SQLPLUS_COMMENT:
+				case SCE_SQL_COMMENTLINEDOC:
+				case SCE_SQL_COMMENTDOCKEYWORD:
+				case SCE_SQL_COMMENTDOCKEYWORDERROR:
 					return TRUE;
 				default:
 					return FALSE;
@@ -1191,6 +1334,8 @@ gboolean sc_speller_is_text(GeanyDocument *doc, gint pos)
 				case SCE_TCL_COMMENT:
 				case SCE_TCL_COMMENTLINE:
 				case SCE_TCL_IN_QUOTE:
+				case SCE_TCL_COMMENT_BOX:
+				case SCE_TCL_BLOCK_COMMENT:
 					return TRUE;
 				default:
 					return FALSE;
@@ -1212,6 +1357,7 @@ gboolean sc_speller_is_text(GeanyDocument *doc, gint pos)
 				case SCE_V_COMMENTLINEBANG:
 				case SCE_V_STRING:
 				case SCE_V_STRINGEOL:
+				case SCE_V_COMMENT_WORD:
 					return TRUE;
 				default:
 					return FALSE;
@@ -1227,6 +1373,7 @@ gboolean sc_speller_is_text(GeanyDocument *doc, gint pos)
 				case SCE_VHDL_COMMENTLINEBANG:
 				case SCE_VHDL_STRING:
 				case SCE_VHDL_STRINGEOL:
+				case SCE_VHDL_BLOCK_COMMENT:
 					return TRUE;
 				default:
 					return FALSE;
