@@ -47,7 +47,7 @@ enum {
     UPDATECHECK_STARTUP
 };
 
-#define UPDATE_CHECK_URL "https://geany.org/service/version.php"
+#define UPDATE_CHECK_URL "https://geany.org/service/version/"
 
 static GtkWidget *main_menu_item = NULL;
 static void update_check_result_cb(SoupSession *session,
@@ -166,16 +166,22 @@ version_compare(const gchar *current_version)
     parse_version_string(current_version, &geany_current.major,
         &geany_current.minor, &geany_current.mini, &geany_current.extra);
 
-    if ((geany_running.major < geany_current.major) ||
-        (geany_running.minor < geany_current.minor) ||
-        (geany_running.minor < geany_current.minor))
-    {
+    if (geany_running.major < geany_current.major)
         return TRUE;
-    }
-    else
-    {
+    if (geany_running.major > geany_current.major)
         return FALSE;
-    }
+
+    if (geany_running.minor < geany_current.minor)
+        return TRUE;
+    if (geany_running.minor > geany_current.minor)
+        return FALSE;
+
+    if (geany_running.mini < geany_current.mini)
+        return TRUE;
+    if (geany_running.mini > geany_current.mini)
+        return FALSE;
+
+    return FALSE;
 }
 
 
@@ -245,10 +251,6 @@ on_configure_response(G_GNUC_UNUSED GtkDialog *dialog, gint response,
         gchar *data;
         gchar *config_dir = g_path_get_dirname(config_file);
 
-        config_file = g_strconcat(geany->app->configdir,
-            G_DIR_SEPARATOR_S, "plugins", G_DIR_SEPARATOR_S,
-            "updatechecker", G_DIR_SEPARATOR_S, "general.conf", NULL);
-
         /* Crabbing options that has been set */
         check_on_startup =
             gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(config_widgets.run_on_startup));
