@@ -20,6 +20,7 @@
 #include "utils.h"
 
 #include "cmds/motion.h"
+#include "cmds/motion-word.h"
 #include "cmds/txtobjs.h"
 #include "cmds/changemode.h"
 #include "cmds/edit.h"
@@ -166,6 +167,21 @@ typedef struct {
 	/* END */
 
 
+/* From the above commands, these commands also include the character
+ * where the destionation movement ends for motion commands (e.g. 'de' will
+ * delete the word including the last character) */
+CmdDef include_dest_char_movement_cmds[] = {
+	{cmd_goto_next_char, GDK_KEY_f, 0, 0, 0, TRUE, FALSE},
+	{cmd_goto_next_char_before, GDK_KEY_t, 0, 0, 0, TRUE, FALSE},
+	{cmd_goto_next_word_end, GDK_KEY_e, 0, 0, 0, FALSE, FALSE},
+	{cmd_goto_next_word_end_space, GDK_KEY_E, 0, 0, 0, FALSE, FALSE},
+	{cmd_goto_previous_word, GDK_KEY_b, 0, 0, 0, FALSE, FALSE},
+	{cmd_goto_previous_word_space, GDK_KEY_B, 0, 0, 0, FALSE, FALSE},
+	{cmd_goto_matching_brace, GDK_KEY_percent, 0, 0, 0, FALSE, FALSE},
+	{NULL, 0, 0, 0, 0, FALSE, FALSE}
+};
+
+
 CmdDef movement_cmds[] = {
 	MOVEMENT_CMDS
 	{NULL, 0, 0, 0, 0, FALSE, FALSE}
@@ -206,6 +222,8 @@ CmdDef operator_cmds[] = {
 	{cmd_select_less, GDK_KEY_a, GDK_KEY_greater, 0, 0, FALSE, FALSE}, \
 	{cmd_select_bracket, GDK_KEY_a, GDK_KEY_bracketleft, 0, 0, FALSE, FALSE}, \
 	{cmd_select_bracket, GDK_KEY_a, GDK_KEY_bracketright, 0, 0, FALSE, FALSE}, \
+	{cmd_select_word, GDK_KEY_a, GDK_KEY_w, 0, 0, FALSE, FALSE}, \
+	{cmd_select_word_space, GDK_KEY_a, GDK_KEY_W, 0, 0, FALSE, FALSE}, \
 	/* inner */ \
 	{cmd_select_quotedbl_inner, GDK_KEY_i, GDK_KEY_quotedbl, 0, 0, FALSE, FALSE}, \
 	{cmd_select_quoteleft_inner, GDK_KEY_i, GDK_KEY_quoteleft, 0, 0, FALSE, FALSE}, \
@@ -220,6 +238,8 @@ CmdDef operator_cmds[] = {
 	{cmd_select_less_inner, GDK_KEY_i, GDK_KEY_greater, 0, 0, FALSE, FALSE}, \
 	{cmd_select_bracket_inner, GDK_KEY_i, GDK_KEY_bracketleft, 0, 0, FALSE, FALSE}, \
 	{cmd_select_bracket_inner, GDK_KEY_i, GDK_KEY_bracketright, 0, 0, FALSE, FALSE}, \
+	{cmd_select_word_inner, GDK_KEY_i, GDK_KEY_w, 0, 0, FALSE, FALSE}, \
+	{cmd_select_word_space_inner, GDK_KEY_i, GDK_KEY_W, 0, 0, FALSE, FALSE}, \
 	/* END */
 
 
@@ -430,7 +450,7 @@ static gboolean is_cmdpart(GSList *kpl, CmdDef *cmds)
 
 static gboolean is_printable(GSList *kpl)
 {
-	guint mask = GDK_MODIFIER_MASK & ~(GDK_SHIFT_MASK | GDK_LOCK_MASK);
+	guint mask = gtk_accelerator_get_default_mod_mask() & ~GDK_SHIFT_MASK;
 	KeyPress *kp = g_slist_nth_data(kpl, 0);
 
 	if (kp->modif & mask)
@@ -506,7 +526,7 @@ static CmdDef *get_cmd_to_run(GSList *kpl, CmdDef *cmds, gboolean have_selection
 				if (cmd->cmd == c)
 					return cmd;
 			}
-			else if (prev && prev->key == GDK_KEY_g)
+			else if (prev && prev->key == GDK_KEY_g && !VI_IS_INSERT(mode))
 			{
 				// takes care of operator commands like g~, gu, gU where we
 				// have no selection yet so the 2-letter command isn't found
@@ -514,7 +534,7 @@ static CmdDef *get_cmd_to_run(GSList *kpl, CmdDef *cmds, gboolean have_selection
 				// would be used instead of waiting for the full command
 			}
 			else if (is_cmdpart(kpl, text_object_cmds) &&
-					get_cmd_to_run(below, operator_cmds, TRUE))
+					get_cmd_to_run(below, operator_cmds, TRUE) && !VI_IS_INSERT(mode))
 			{
 				// if we received "a" or "i", we have to check if there's not
 				// an operator command below because these can be part of
@@ -566,7 +586,8 @@ static void perform_cmd(CmdDef *def, CmdContext *ctx)
 	if (VI_IS_COMMAND(vi_get_mode()))
 	{
 		gboolean is_text_object_cmd = is_in_cmd_group(text_object_cmds, def);
-		if (is_text_object_cmd ||is_in_cmd_group(movement_cmds, def))
+		gboolean is_include_dest_char_movement_cmd = is_in_cmd_group(include_dest_char_movement_cmds, def);
+		if (is_text_object_cmd || is_in_cmd_group(movement_cmds, def))
 		{
 			def = get_cmd_to_run(top, operator_cmds, TRUE);
 			if (def)
@@ -584,6 +605,12 @@ static void perform_cmd(CmdDef *def, CmdContext *ctx)
 				{
 					sel_start = MIN(new_pos, orig_pos);
 					sel_len = ABS(new_pos - orig_pos);
+					if (sel_len > 0 && is_include_dest_char_movement_cmd)
+					{
+						sel_len++;
+						if (new_pos < orig_pos)
+							sel_start--;
+					}
 				}
 				cmd_params_init(&param, ctx->sci,
 					1, FALSE, top, TRUE,
@@ -610,14 +637,25 @@ static gboolean perform_repeat_cmd(CmdContext *ctx)
 	gint i;
 
 	def = get_cmd_to_run(ctx->repeat_kpl, edit_cmds, FALSE);
-	if (!def)
-		return FALSE;
-
 	num = num == -1 ? 1 : num;
-	for (i = 0; i < num; i++)
-		perform_cmd(def, ctx);
+	if (def) {
+		for (i = 0; i < num; i++)
+			perform_cmd(def, ctx);
+		return TRUE;
+	}
+	else if (ctx->insert_buf_len > 0) {
+		gint pos;
 
-	return TRUE;
+		SSM(ctx->sci, SCI_BEGINUNDOACTION, 0, 0);
+		for (i = 0; i < num; i++)
+			SSM(ctx->sci, SCI_ADDTEXT, ctx->insert_buf_len, (sptr_t) ctx->insert_buf);
+		pos = SSM(ctx->sci, SCI_GETCURRENTPOS, 0, 0);
+		SET_POS(ctx->sci, PREV(ctx->sci, pos), FALSE);
+		SSM(ctx->sci, SCI_ENDUNDOACTION, 0, 0);
+		return TRUE;
+	}
+
+	return FALSE;
 }
 
 
