@@ -284,6 +284,16 @@ key_file_set_value (GKeyFile     *kf,
       g_key_file_set_string (kf, group, key, g_value_get_string (value));
       break;
     
+    case G_TYPE_BOXED:
+      if (G_VALUE_HOLDS (value, G_TYPE_STRV)) {
+        gchar **val = g_value_get_boxed (value);
+        
+        g_key_file_set_string_list (kf, group, key, (const gchar *const *) val,
+                                    val ? g_strv_length (val) : 0);
+        break;
+      }
+      /* fallthrough */
+    
     default:
       g_set_error (error, G_KEY_FILE_ERROR, G_KEY_FILE_ERROR_INVALID_VALUE,
                    "Unsupported setting type \"%s\" for setting \"%s::%s\"",
@@ -404,6 +414,18 @@ key_file_get_value (GKeyFile     *kf,
       break;
     }
     
+    case G_TYPE_BOXED:
+      if (G_VALUE_HOLDS (value, G_TYPE_STRV)) {
+        gchar **val;
+        
+        val = g_key_file_get_string_list (kf, group, key, NULL, &err);
+        if (! err) {
+          g_value_take_boxed (value, val);
+        }
+        break;
+      }
+      /* fallthrough */
+    
     default:
       g_set_error (&err, G_KEY_FILE_ERROR, G_KEY_FILE_ERROR_INVALID_VALUE,
                    "Unsupported setting type \"%s\" for setting \"%s::%s\"",
@@ -619,14 +641,14 @@ gwh_settings_widget_int_new (GwhSettings  *self,
                              gboolean     *needs_label)
 {
   GtkWidget      *button;
-  GtkObject      *adj;
+  GtkAdjustment  *adj;
   GParamSpecInt  *pspec_int = G_PARAM_SPEC_INT (pspec);
   
   adj = gtk_adjustment_new ((gdouble)g_value_get_int (value),
                             (gdouble)pspec_int->minimum,
                             (gdouble)pspec_int->maximum,
                             1.0, 10.0, 0.0);
-  button = gtk_spin_button_new (GTK_ADJUSTMENT (adj), 0.0, 0);
+  button = gtk_spin_button_new (adj, 0.0, 0);
   *needs_label = TRUE;
   
   return button;
@@ -771,7 +793,7 @@ gwh_settings_widget_new_full (GwhSettings            *self,
       GtkWidget *box;
       gchar     *label;
       
-      box = gtk_hbox_new (FALSE, 6);
+      box = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 6);
       label = g_strdup_printf (_("%s:"), g_param_spec_get_nick (pspec));
       gtk_box_pack_start (GTK_BOX (box), gtk_label_new (label), FALSE, TRUE, 0);
       g_free (label);
