@@ -27,6 +27,7 @@
 #include "dialogs.h"
 #include "menu.h"
 #include "sidebar.h"
+#include "search_projects.h"
 
 extern GeanyPlugin *geany_plugin;
 
@@ -36,8 +37,8 @@ typedef struct
 	GtkWidget *root_item;
 	GtkWidget *item_new;
 	GtkWidget *item_open;
-	GtkWidget *item_save;
 	GtkWidget *item_settings;
+	GtkWidget *item_search_projects;
 	GtkWidget *item_close;
 }WB_MENU_DATA;
 static WB_MENU_DATA menu_data;
@@ -58,15 +59,22 @@ void menu_set_context(MENU_CONTEXT context)
 		case MENU_CONTEXT_WB_OPENED:
 			gtk_widget_set_sensitive(menu_data.item_new, FALSE);
 			gtk_widget_set_sensitive(menu_data.item_open, FALSE);
-			gtk_widget_set_sensitive(menu_data.item_save, TRUE);
 			gtk_widget_set_sensitive(menu_data.item_settings, TRUE);
+			gtk_widget_set_sensitive(menu_data.item_search_projects, TRUE);
 			gtk_widget_set_sensitive(menu_data.item_close, TRUE);
 		break;
+		case MENU_CONTEXT_SEARCH_PROJECTS_SCANING:
+			gtk_widget_set_sensitive(menu_data.item_new, FALSE);
+			gtk_widget_set_sensitive(menu_data.item_open, FALSE);
+			gtk_widget_set_sensitive(menu_data.item_settings, TRUE);
+			gtk_widget_set_sensitive(menu_data.item_search_projects, FALSE);
+			gtk_widget_set_sensitive(menu_data.item_close, FALSE);
+		break;
 		case MENU_CONTEXT_WB_CLOSED:
 			gtk_widget_set_sensitive(menu_data.item_new, TRUE);
 			gtk_widget_set_sensitive(menu_data.item_open, TRUE);
-			gtk_widget_set_sensitive(menu_data.item_save, FALSE);
 			gtk_widget_set_sensitive(menu_data.item_settings, FALSE);
+			gtk_widget_set_sensitive(menu_data.item_search_projects, FALSE);
 			gtk_widget_set_sensitive(menu_data.item_close, FALSE);
 		break;
 	}
@@ -127,22 +135,6 @@ static void item_open_workbench_activate_cb(G_GNUC_UNUSED GtkMenuItem *menuitem,
 }
 
 
-/* The function handles the menu item "Save workbench" */
-static void item_save_workbench_activate_cb(G_GNUC_UNUSED GtkMenuItem *menuitem, G_GNUC_UNUSED gpointer user_data)
-{
-	GError *error = NULL;
-
-	if (wb_globals.opened_wb != NULL)
-	{
-		if (!workbench_save(wb_globals.opened_wb, &error))
-		{
-			dialogs_show_msgbox(GTK_MESSAGE_INFO, _("Could not save workbench file: %s"), error->message);
-		}
-		sidebar_update(SIDEBAR_CONTEXT_WB_SAVED, NULL);
-	}
-}
-
-
 /* The function handles the menu item "Settings" */
 static void item_workbench_settings_activate_cb(G_GNUC_UNUSED GtkMenuItem *menuitem, G_GNUC_UNUSED gpointer user_data)
 {
@@ -153,8 +145,17 @@ static void item_workbench_settings_activate_cb(G_GNUC_UNUSED GtkMenuItem *menui
 		enable_live_update_old = workbench_get_enable_live_update(wb_globals.opened_wb);
 		if (dialogs_workbench_settings(wb_globals.opened_wb))
 		{
+			GError *error = NULL;
+
 			sidebar_update(SIDEBAR_CONTEXT_WB_SETTINGS_CHANGED, NULL);
 
+			/* Save the new settings to the workbench file (.geanywb). */
+			if (!workbench_save(wb_globals.opened_wb, &error))
+			{
+				dialogs_show_msgbox(GTK_MESSAGE_INFO, _("Could not save workbench file: %s"), error->message);
+			}
+			sidebar_update(SIDEBAR_CONTEXT_WB_SAVED, NULL);
+
 			enable_live_update = workbench_get_enable_live_update(wb_globals.opened_wb);
 			if (enable_live_update != enable_live_update_old)
 			{
@@ -174,6 +175,17 @@ static void item_workbench_settings_activate_cb(G_GNUC_UNUSED GtkMenuItem *menui
 }
 
 
+/* The function handles the menu item "Search projects" */
+static void item_workbench_search_projects_activate_cb(G_GNUC_UNUSED GtkMenuItem *menuitem, G_GNUC_UNUSED gpointer user_data)
+{
+	if (wb_globals.opened_wb != NULL)
+	{
+		search_projects(wb_globals.opened_wb);
+	}
+	sidebar_update(SIDEBAR_CONTEXT_PROJECT_ADDED, NULL);
+}
+
+
 /* The function handles the menu item "Close workbench" */
 static void item_close_workbench_activate_cb(G_GNUC_UNUSED GtkMenuItem *menuitem, G_GNUC_UNUSED gpointer user_data)
 {
@@ -209,13 +221,6 @@ gboolean menu_init(void)
 	g_signal_connect(menu_data.item_open, "activate",
 					 G_CALLBACK(item_open_workbench_activate_cb), NULL);
 
-	/* Create new menu item "Save Workbench" */
-	menu_data.item_save = gtk_menu_item_new_with_mnemonic(_("_Save"));
-	gtk_widget_show(menu_data.item_save);
-	gtk_menu_shell_append(GTK_MENU_SHELL (menu_data.menu), menu_data.item_save);
-	g_signal_connect(menu_data.item_save, "activate",
-					 G_CALLBACK(item_save_workbench_activate_cb), NULL);
-
 	/* Create new menu item "Workbench Settings" */
 	menu_data.item_settings = gtk_menu_item_new_with_mnemonic(_("S_ettings"));
 	gtk_widget_show(menu_data.item_settings);
@@ -223,6 +228,13 @@ gboolean menu_init(void)
 	g_signal_connect(menu_data.item_settings, "activate",
 					 G_CALLBACK(item_workbench_settings_activate_cb), NULL);
 
+	/* Create new menu item "Search Projects" */
+	menu_data.item_search_projects = gtk_menu_item_new_with_mnemonic(_("Search _projects"));
+	gtk_widget_show(menu_data.item_search_projects);
+	gtk_menu_shell_append(GTK_MENU_SHELL (menu_data.menu), menu_data.item_search_projects);
+	g_signal_connect(menu_data.item_search_projects, "activate",
+					 G_CALLBACK(item_workbench_search_projects_activate_cb), NULL);
+
 	/* Create new menu item "Close Workbench" */
 	menu_data.item_close = gtk_menu_item_new_with_mnemonic(_("_Close"));
 	gtk_widget_show(menu_data.item_close);
