@@ -28,6 +28,7 @@
 #endif
 #include "wb_globals.h"
 #include <gtkcompat.h>
+#include <gp_gtkcompat.h>
 
 #include "sidebar.h"
 #include "popup_menu.h"
@@ -446,6 +447,7 @@ static void sidebar_add_file (WB_PROJECT *prj, WB_PROJECT_DIR *root, const gchar
 	{
 		g_object_unref(icon);
 	}
+	g_free(name);
 }
 
 
@@ -549,11 +551,14 @@ static gboolean sidebar_get_filepath_iter (WB_PROJECT *prj, WB_PROJECT_DIR *root
 	part = g_strdup(&(filepath[len]));
 	if (strlen(part) == 0)
 	{
+		g_free(part);
 		return FALSE;
 	}
 	parts = g_strsplit(part, G_DIR_SEPARATOR_S, -1);
 	if (parts[0] == NULL)
 	{
+		g_free(part);
+		g_strfreev(parts);
 		return FALSE;
 	}
 
@@ -866,6 +871,9 @@ static void sidebar_update_workbench(GtkTreeIter *iter, gint *position)
 		gtk_tree_view_set_hover_expand(GTK_TREE_VIEW(sidebar.file_view),
 			workbench_get_expand_on_hover(wb_globals.opened_wb));
 
+		gtk_tree_view_set_enable_tree_lines(GTK_TREE_VIEW(sidebar.file_view),
+			workbench_get_enable_tree_lines(wb_globals.opened_wb));
+
 		count = workbench_get_project_count(wb_globals.opened_wb);
 		length = g_snprintf(text, sizeof(text),
 							g_dngettext(GETTEXT_PACKAGE, "%s: %u Project", "%s: %u Projects", count),
@@ -879,7 +887,8 @@ static void sidebar_update_workbench(GtkTreeIter *iter, gint *position)
 		if (count == 0)
 		{
 			gtk_tree_store_clear(sidebar.file_store);
-			sidebar_show_intro_message(_("Add a project\nusing the context menu."), TRUE);
+			sidebar_show_intro_message(_("Add a project using the context menu\n"
+				"or select \"Search projects\" from the menu."), TRUE);
 		}
 		else
 		{
@@ -1373,6 +1382,7 @@ void sidebar_init(void)
 	GList *focus_chain = NULL;
 
 	sidebar.file_view_vbox = gtk_vbox_new(FALSE, 0);
+	gtk_widget_set_name(sidebar.file_view_vbox, "workbench");
 
 	/**** label ****/
 	sidebar.file_view_label = gtk_label_new (_("No workbench opened."));
